unit ZoomBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  extctrls, menus;

type
  EInvalidZoomLevel = Class(Exception);

  TZoomBox = class(TScrollBox)
  private
    FMultiplier, FDefaultMultiplier : Extended;   
    StartX, StartY, EndX, EndY : Integer;
    ZoomBegun : Boolean;
    FMargin: integer;
    FMinX: extended;
    FMaxY: extended;
    FMaxX: extended;
    FMinY: extended;
    FPaintBox: TPaintBox;
    FZoomWindow: TShape;
    PointsList : TList;
    FOnClick: TNotifyEvent;
    FOnDblClick : TNotifyEvent;
    FOnExit : TNotifyEvent;
    FOnDragDrop : TDragDropEvent;
    FOnDragOver: TDragOverEvent;
    FOnEndDrag: TEndDragEvent;
    FOnEndDock: TEndDragEvent;
    FOnMouseDown: TMouseEvent;
    FOnMouseMove: TMouseMoveEvent;
    FOnMouseUp: TMouseEvent;
    FOnStartDrag: TStartDragEvent;
    procedure SetMultiplier( Value: extended);
    procedure SetMargin(const Value: integer);
    procedure SetMaxX(const Value: extended);
    procedure SetMaxY(const Value: extended);
    procedure SetMinX(const Value: extended);
    procedure SetMinY(const Value: extended);
    function ReadPBCanvas : TCanvas;
    function ReadPBClientHeight: Integer;
    function ReadPBClientWidth: Integer;
    function GetPBClientOrigin : TPoint;
    function GetPBClientRect : TRect;
    function GetPBColor : TColor;
    procedure SetPBColor(const Value: TColor);
    function GetPBComponentCount : integer;
    function GetPBComponentIndex : integer;
    procedure SetPBComponentIndex(const Value: integer);
    function GetPBComponents (Index : integer) : TComponent;
    function GetPBControlState : TControlState;
    procedure SetPBControlState (const Controlstate: TControlState);
    function GetPBControlStyle : TControlStyle;
    procedure SetPBControlStyle (const ControlStyle: TControlStyle);
    function GetPBCursor : TCursor;
    procedure SetPBCursor (const Cursor: TCursor);
    function GetPBDragKind : TDragKind;
    procedure SetPBDragKind(const Value : TDragKind);
    function GetPBDragCursor: TCursor;
    procedure SetPBDragCursor(const Value: TCursor);
    function GetPBDragMode: TDragMode;
    procedure SetPBDragMode(const Value: TDragMode);
    function GetPBEnabled: Boolean;
    procedure SetPBEnabled(const Value: Boolean);
    function GetPBFont: TFont;
    procedure SetPBFont(const Value: TFont);
    function GetPBHeight: Integer;
    procedure SetPBHeight(const Value: Integer);
    function GetPBHint: string;
    procedure SetPBHint(const Value: string);
    function GetPBLeft: Integer;
    procedure SetPBLeft(const Value: Integer);
    function GetPBPopupMenu: TPopupMenu;
    procedure SetPBPopupMenu(const Value: TPopupMenu);
    function GetPBShowHint: Boolean;
    procedure SetPBShowHint(const Value: Boolean);
    function GetPBTop: Integer;
    procedure SetPBTop(const Value: Integer);
    function GetPBVisible: Boolean;
    procedure SetPBVisible(const Value: Boolean);
    function GetPBWidth: Integer;
    procedure SetPBWidth(const Value: Integer);
    function GetPBWindowProc: TWndMethod;
    procedure SetPBWindowProc(const Value: TWndMethod);
    function GetPBComObject : IUnknown;
    function GetPBTag: Longint;
    procedure SetPBTag(const Value: Longint);
    function GetSBrush: TBrush;
    procedure SetSBrush(const Value: TBrush);
    function GetSPen: TPen;
    procedure SetSPen(const Value: TPen);
    function GetSBoundsRect: TRect;
    procedure SetSBoundsRect(const Value: TRect);
    function GetSCursor: TCursor;
    procedure SetSCursor(const Value: TCursor);
    function GetSDragCursor: TCursor;
    procedure SetSDragCursor(const Value: TCursor);
    function GetOnPBPaint: TNotifyEvent;
    procedure SetOnPBPaint(const Value: TNotifyEvent);
//    function GetOnClick: TNotifyEvent;
    procedure SetOnClick(const Value: TNotifyEvent);
//    function GetOnPBDblClick: TNotifyEvent;
    procedure SetOnDblClick(const Value: TNotifyEvent);
//    function GetOnPBDragDrop: TDragDropEvent;
    procedure SetOnDragDrop(const Value: TDragDropEvent);
//    function GetOnPBDragOver: TDragOverEvent;
    procedure SetOnDragOver(const Value: TDragOverEvent);
//    function GetOnBEndDrag: TEndDragEvent;
    procedure SetOnEndDrag(const Value: TEndDragEvent);
//    function GetOnPBMouseDown: TMouseEvent;
    procedure SetOnEndDrop(const Value: TEndDragEvent);
//    function GetOnPBMouseDown: TMouseEvent;
    procedure SetOnMouseDown(const Value: TMouseEvent);
//    function GetOnPBMouseMove: TMouseMoveEvent;
    procedure SetOnMouseMove(const Value: TMouseMoveEvent);
//    function GetOnPBMouseUp: TMouseEvent;
    procedure SetOnMouseUp(const Value: TMouseEvent);
//    function GetOnPBStartDrag: TStartDragEvent;
    procedure SetOnStartDrag(const Value: TStartDragEvent);
    { Private declarations }
  protected
    procedure CreateWnd; override ;
    {Creates a Windows control corresponding to the TZoomBox object.
    Description
    Use CreateWnd to create a Windows control corresponding to the
    ZoomBox object.
    CreateWnd initializes the paintbox size
    and then calls TScrollingWinControl.CreateWnd.}
    procedure DoExit; override;
    {DoExit  calls AbortZoom and then calls TWinControl.DoExit}
    { Protected declarations }
  public
    Constructor Create(AOwner: TComponent); Override;
    {Creates and initializes a new TZoomBox object.
    Description
    Create performs the following tasks:
    Calls the TScrollBox Create method, passing it AOwner.
    Creates a Paintbox and shape passing itself as AOwner.
    It Creates a list of TZoomPoints.
    Sets the Margin to 20.
    Sets Multiplier to 1.}
    Destructor Destroy; override;
    {Removes the TScrollingWinControl object from memory.
    Description
    Do not call Destroy. Instead, call Free, which calls Destroy if the
    TZoomBox object is not nil.
    Destroy frees the Paintbox and shape and the list of TZoomPoints and then
    calls TScrollBox.Destroy.
    It does not destroy the TZoomPoints in the list.}
    procedure AbortZoom;
    {If for some reason you want to terminate a zoom operation without doing
    the zoom, call AbortZoom instead of BeginZoom.}
    procedure BeginZoom( X, Y: Integer);
    {Begin Zoom starts a zooming operation. X, and Y are the X and Y screen
    coordinates of the Paintbox in the TZoomBox. You can those from
    OnPBMouseDown.}
    procedure ContinueZoom ( X, Y: Integer);
    {If BeginZoom has been called, and neither AbortZoom nor FinishZoom have
    yet been called, ContinueZoom will draw a shape on the paintbox.
    One of the corners of the shape will be at the location where the zoom
    began. The other will be at X, and Y where are the X and Y screen
    coordinates of the Paintbox in the TZoomBox. You can those from
    OnPBMouseMove.}
    procedure FinishZoom ( X, Y: Integer);
    {FinishZoom terminates a zoom operation. X, and Y are the X and Y screen
    coordinates of the Paintbox in the TZoomBox. You can those from
    OnPBMouseUp. FinishZoom sets the zoom level so that
    the area outlined by the X, and Y parameters in BeginZoom and the
    X, and Y parameters in FinishZoom are displated in the TZoomBox. }
    procedure GetMinMax;
    {GetMinMax determines the minimum and maximum coordinates of the currently
    existing TZoomPoints associated with the TZoomBox and sets the
    DefaultMultiplier to a value sufficient to display all the TZoomPoints and
    still leave a blank area equal to Margin pixels on all sides.}
    procedure MouseToCoordinates(XCoord, YCoord : integer; var X, Y : extended);
    {MouseToCoordinates changes the screen coordinates XCoord and YCoord to the
    floating point coordinates X and Y.}
    function SetZoom(AZoomLevel: Extended) : Extended;
    {SetZoom changes the degree of zooming. AZoomLevel should be the degree
    of zooming that you wish to set. If AZoomLevel  is set equal to
    DefaultMultiplier, this is equivalent to ZoomOut. The zoom level that
    is actually set will generally differ slightly from that specified by
    AZoomLevel. However, the displayed region will differ from that requested by
    one pixel or less. The actuall zoom level that is set is the result of the
    SetZoom function. }
    Procedure ZoomBy(ZoomFactor : Extended);
    {ZoomBy changes the level of zoom by a factor specified by ZoomFactor. The
    Screen coordinates are not changed by ZoomBy. See also ZoomByAt}
    Procedure ZoomByAt(ZoomFactor, X, Y : Extended);
    {ZoomByAt calls ZoomBy and passes it ZoomFactor. It then centers the
    position indicated by the floating point coordinates X and Y.}
    procedure ZoomOut;
    {ZoomOut sets the zoom level so that all the points that had been created
    at the time of the previous call to GetMinMax will be displayed with a
    blank area at the edge with a width equal to Margin.}

    procedure PBBeginDrag(Immediate: Boolean; Threshold: Integer = -1);
    {See TPaintBox.BeginDrag.}
    procedure PBBringToFront;
    property PBCanvas : TCanvas read ReadPBCanvas ;
    {See TPaintBox.Canvas.}
    property PBClientHeight :Integer read ReadPBClientHeight ;
    {See TPaintBox.ClientHeight.}
    property PBClientOrigin : TPoint read GetPBClientOrigin;
    {See TPaintBox.ClientOrigin.}
    property PBClientRect : TRect read GetPBClientRect;
    {See TPaintBox.ClientRect.}
    function PBClientToScreen(const Point: TPoint): TPoint;
    {See TPaintBox.ClientToScreen.}
    property PBClientWidth :Integer read ReadPBClientWidth ;
    {See TPaintBox.ClientWidth.}
    property PBComponentCount : integer read GetPBComponentCount;
    {See TPaintBox.ComponentCount.}
    property PBComponentIndex : integer read GetPBComponentIndex
       write SetPBComponentIndex;
    {See TPaintBox.ComponentIndex.}
    property PBComponents[Index: Integer]: TComponent read GetPBComponents;
    {See TPaintBox.Components.}
    property PBComObject : IUnknown read GetPBComObject;
    {See TPaintBox.ComObject.}
    property PBControlState : TControlState read GetPBControlState
       write SetPBControlState;
    {See TPaintBox.ControlState.}
    property PBControlStyle: TControlStyle read GetPBControlStyle
       write SetPBControlStyle;
    {See TPaintBox.ControlStyle.}
    procedure PBDragDrop(Source: TObject; X, Y: Integer);
    {See TPaintBox.DragDrop}
    function PBDragging: Boolean;
    procedure PBEndDrag(Drop: Boolean);
    function PBGetTextBuf(Buffer: PChar; BufSize: Integer): Integer;
    function PBGetTextLen: Integer;
    procedure PBHide;
    function PBPerform(Msg: Cardinal; WParam, LParam: Longint): Longint;
    procedure PbRefresh;
    procedure PBRepaint;
    function PBScreenToClient(const Point: TPoint): TPoint;
    procedure PBSendToBack;
    procedure PBSetBounds(ALeft, ATop, AWidth, AHeight: Integer);
    procedure PBSetTextBuf(Buffer: PChar);
    procedure PBShow;
    procedure PBUpdate;
    property PBWindowProc: TWndMethod read GetPBWindowProc
       write SetPBWindowProc;
    {See TPaintBox.WindowProc.}
    property SBoundsRect: TRect read GetSBoundsRect write SetSBoundsRect;
    {See TShape.BoundsRect.}
    Property DefaultMultiplier : Extended read FDefaultMultiplier;
    {DefaultMultiplier is the zoom level that would display all the TZoomPoints
    with a blank area surrounding them Margin pixels wide
    that were in existence when the last GetMinMax occurred.}
    procedure Invalidate; override;
    procedure SetRange(MinimumX, MaximumX, MinimumY, MaximumY: Extended);
    procedure SetXRange(MinimumX, MaximumX: Extended);
    procedure SetYRange(MinimumY, MaximumY: Extended);
    property PBHeight : Integer read GetPBHeight write SetPBHeight;
    {See TPaintBox.Height.}
    property PBWidth : Integer read GetPBWidth write SetPBWidth;
    {See TPaintBox.Width.}
    { Public declarations }
  published
    Property Margin : integer read FMargin write SetMargin;
    {The TZoomBox maintains a blank area at least Margin pixels wide on all
    sides of the TZoomPoints that were in existence the last time GetMinMax was
    called.}
    Property MinX : extended read FMinX write SetMinX;
    {MinX is the minimum X value of all the TZoomPoints that were in existence
    the last time GetMinMax was
    called. If no TZoomPoints were in existence, it is set to a default value.}
    Property MaxX : extended read FMaxX write SetMaxX;
    {MaxX is the maximum X value of all the TZoomPoints that were in existence
    the last time GetMinMax was
    called. If no TZoomPoints were in existence, it is set to a default value.}
    Property MinY : extended read FMinY write SetMinY;
    {MinX is the minimum Y value of all the TZoomPoints that were in existence
    the last time GetMinMax was
    called. If no TZoomPoints were in existence, it is set to a default value.}
    Property MaxY : extended read FMaxY write SetMaxY;
    {MaxX is the maximum Y value of all the TZoomPoints that were in existence
    the last time GetMinMax was
    called. If no TZoomPoints were in existence, it is set to a default value.}
    property OnExit: TNotifyEvent read FOnExit write FOnExit;
    {OnExit occurs when the focus passes away from the TZoomBox.}
    property OnClick: TNotifyEvent read FOnClick write SetOnClick;
    {See TPaintBox.OnClick.}
    property OnDblClick: TNotifyEvent read FOnDblClick write SetOnDblClick;
    {See TPaintBox.OnDblClick.}
    property OnDragDrop: TDragDropEvent read FOnDragDrop write SetOnDragDrop;
    {See TPaintBox.OnDragDrop.}
    property OnDragOver: TDragOverEvent read FOnDragOver write SetOnDragOver;
    {See TPaintBox.OnDragOver.}
    property OnEndDrag: TEndDragEvent read FOnEndDrag write SetOnEndDrag;
    {See TPaintBox.OnEndDrag.}
    property FnEndDrag: TEndDragEvent read FOnEndDrag write SetOnEndDrop;
    Property Multiplier : Extended read FMultiplier write SetMultiplier;
    {Multiplier is the current zoom level.}
    property OnMouseDown: TMouseEvent read FOnMouseDown write SetOnMouseDown;
    {See TPaintBox.OnMouseDown.}
    property OnMouseMove: TMouseMoveEvent read FOnMouseMove write SetOnMouseMove;
    {See TPaintBox.OnMouseMove.}
    property OnMouseUp: TMouseEvent read FOnMouseUp write SetOnMouseUp;
    {See TPaintBox.OnMouseUp.}
    property OnPaint: TNotifyEvent read GetOnPBPaint write SetOnPBPaint;
    {See TPaintBox.OnPaint.}
    property OnStartDrag: TStartDragEvent read FOnStartDrag write SetOnStartDrag;
    {See TPaintBox.OnStartDrag.}
    property PBColor : TColor read GetPBColor Write SetPBColor;
    {See TPaintBox.Color.}
    property PBCursor : TCursor read GetPBCursor Write SetPBCursor;
    {See TPaintBox.Cursor.}
    property PBDragCursor : TCursor read GetPBDragCursor Write SetPBDragCursor;
    {See TPaintBox.DragCursor.}
    property PBDragKind: TDragKind read GetPBDragKind Write SetPBDragKind;
    {See TPaintBox.DragKind.}
    property PBDragMode: TDragMode read GetPBDragMode Write SetPBDragMode;
    {See TPaintBox.DragMode.}
    property PBEnabled: Boolean read GetPBEnabled Write SetPBEnabled;
    {See TPaintBox.Enabled.}
    property PBFont : TFont read GetPBFont write SetPBFont;
    {See TPaintBox.Font.}
    property PBHint : string read GetPBHint write SetPBHint;
    {See TPaintBox.Hint.}
    property PBLeft : integer read GetPBLeft write SetPBLeft;
    {See TPaintBox.Left.}
    property PBPopupMenu : TPopupMenu read GetPBPopupMenu write SetPBPopupMenu;
    {See TPaintBox.PopupMenu.}
    property PBShowHint: Boolean read GetPBShowHint write SetPBShowHint;
    {See TPaintBox.ShowHint.}
    property PBTag : Longint read GetPBTag write SetPBTag;
    {See TPaintBox.Tag.}
    property PBTop: Integer read GetPBTop write SetPBTop;
    {See TPaintBox.Top.}
    property PBVisible: Boolean read GetPBVisible write SetPBVisible;
    {See TPaintBox.Visible.}
    property SBrush: TBrush read GetSBrush write SetSBrush;
    {See TShape.Brush.}
    property SCursor: TCursor read GetSCursor write SetSCursor;
    {See TShape.Cursor.}
    property SDragCursor: TCursor read GetSDragCursor write SetSDragCursor;
    {See TShape.DragCursor.}
    property SPen:  TPen read GetSPen write SetSPen;
    {See TShape.Pen.}
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    { Published declarations }
  end;

  TZoomPoint = class(TObject)
  private
    FY: Extended;
    FX: Extended;
    Parent : TZoomBox;
    function GetXCoord : Integer;
    function GetYCoord : Integer;
    Procedure SetXCoord(AnX : Integer);
    Procedure SetYCoord(AY  : Integer);
    procedure SetX(const Value: Extended);
    procedure SetY(const Value: Extended);
  public
//    function PointToString : string;
//    procedure StringToPoint(AString : string);
    Constructor Create (AZoomImage : TRBWZoomBox);
    Destructor Destroy ; override;
    Property XCoord : Integer read GetXCoord write SetXCoord;
    property YCoord : Integer read GetYCoord write SetYCoord;
    Property X : Extended read FX write SetX;
    Property Y : Extended read FY write SetY;
//    Procedure Draw;
//    function Selected(AnX, AY : integer) : boolean;
  end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TZoomBox]);
end;

{ TZoomBox }

procedure TZoomBox.BeginZoom(X, Y: Integer);
begin
    ZoomBegun := True;
    StartX := X - HorzScrollBar.Position;
    StartY := Y - VertScrollBar.Position;

end;

procedure TZoomBox.ContinueZoom(X, Y: Integer);
begin

  if ZoomBegun then
  begin
    EndX := X - HorzScrollBar.Position;
    EndY := Y - VertScrollBar.Position;
    if StartX < EndX then
    begin
      FZoomWindow.Left := StartX;
      FZoomWindow.Width := EndX - StartX;
    end
    else
    begin
      FZoomWindow.Left := EndX;
      FZoomWindow.Width := StartX - EndX;
    end;
    if StartY < EndY then
    begin
      FZoomWindow.Top := StartY;
      FZoomWindow.Height := EndY - StartY;
    end
    else
    begin
      FZoomWindow.Top := EndY;
      FZoomWindow.Height := StartY - EndY;
    end;
  end;

end;

constructor TZoomBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  PointsList := TList.Create;
  FMaxX := 1;
  FMaxY := 1;
  FMargin := 20;
  FPaintBox := TPaintBox.Create(self);
  FPaintBox.Parent := self;
  FZoomWindow := TShape.Create(self);
  FZoomWindow.Parent := self;
  FZoomWindow.Width := 0;
  FZoomWindow.Height := 0;
  FZoomWindow.Brush.Style := bsClear;
  FMultiplier := 1;
end;

procedure TZoomBox.CreateWnd;
begin
  inherited CreateWnd;
  FPaintBox.Width := Round(ClientWidth*FMultiplier);
  FPaintBox.Height := Round(ClientHeight*FMultiplier);
end;

procedure TZoomBox.FinishZoom(X, Y: Integer);
var
  XMult, YMult, LocalMult : Extended;
  LeftX, RightX, TopY, BottomY : integer;
  APoint : TZoomPoint;
  Width, Height : Integer;
begin
  ZoomBegun := False;
  FZoomWindow.Width := 0;
  FZoomWindow.Height := 0;
  FPaintbox.Cursor := crDefault;
  EndX := X - HorzScrollBar.Position;
  EndY := Y - VertScrollBar.Position;

  if StartX < EndX then
  begin
    LeftX := StartX + HorzScrollBar.Position;
    RightX := EndX + HorzScrollBar.Position ;
  end
  else
  begin
    LeftX := EndX + HorzScrollBar.Position;
    RightX := StartX + HorzScrollBar.Position;
  end;
  if StartY < EndY then
  begin
    TopY := StartY + VertScrollBar.Position;
    BottomY := EndY + VertScrollBar.Position;
  end
  else
  begin
    TopY := EndY + VertScrollBar.Position;
    BottomY := StartY + VertScrollBar.Position;
  end;

  if (RightX > LeftX) and (BottomY > TopY) then
  begin
    APoint := TZoomPoint.Create(self);
    try
      begin

        XMult := (ClientWidth*FMultiplier )/(RightX-LeftX);
        YMult := (ClientHeight*FMultiplier)/(BottomY-TopY);

        APoint.XCoord := LeftX ;
        APoint.YCoord := TopY;

        if XMult < YMult then
        begin
          LocalMult := XMult;
        end
        else
        begin
          LocalMult := YMult;
        end;
        Width := 2*Margin + Round((MaxX-MinX)*LocalMult);
        if Width > High(Smallint) then
        begin
          LocalMult := (High(Smallint) -1 - 2* Margin)/(MaxX-MinX);
          Width := 2*Margin + Round((MaxX-MinX)*LocalMult);
        end;
        Height := 2*Margin + Round((MaxY-MinY)*LocalMult);
        if Height > High(Smallint) then
        begin
          LocalMult := (High(Smallint) -1 - 2* Margin)/(MaxY-MinY);
          Width := 2*Margin + Round((MaxX-MinX)*LocalMult);
          Height := 2*Margin + Round((MaxY-MinY)*LocalMult);
        end;

        FPaintbox.Width := Width;
        FPaintbox.Height := Height;
        FMultiplier := LocalMult;
        HorzScrollBar.Range := FPaintbox.Width;
        VertScrollBar.Range := FPaintbox.Height;
        HorzScrollBar.Position := APoint.XCoord;
        VertScrollBar.Position := APoint.YCoord;
        FPaintbox.Invalidate;
      end;
    finally
      begin
        APoint.Free;
      end;
    end;
  end;
end;



procedure TZoomBox.SetMargin(const Value: integer);
begin
  FMargin := Value;
end;

procedure TZoomBox.SetMaxX(const Value: extended);
begin
  FMaxX := Value;
  if FMinX >= FMaxX then
  begin
    FMinX := FMaxX -1;
  end;
end;

procedure TZoomBox.SetMaxY(const Value: extended);
begin
  FMaxY := Value;
  if FMinY >= FMaxY then
  begin
    FMinY := FMaxY -1;
  end;
end;

procedure TZoomBox.SetMinX(const Value: extended);
begin
  FMinX := Value;
  if FMinX >= FMaxX then
  begin
    FMaxX := FMinX +1;
  end;
end;

procedure TZoomBox.SetMinY(const Value: extended);
begin
  FMinY := Value;
  if FMinY >= FMaxY then
  begin
    FMaxY := FMinY +1;
  end;
end;



function TZoomBox.ReadPBCanvas : TCanvas;
begin
  result := FPaintBox.Canvas ;
end;

{ TPoint }

constructor TZoomPoint.Create(AZoomImage: TRBWZoomBox);
begin
  Parent := AZoomImage;
  AZoomImage.PointsList.Add(self);
end;

destructor TZoomPoint.Destroy;
begin
  Parent.PointsList.Remove(self);
  inherited Destroy;
end;

function TZoomPoint.GetXCoord: Integer;
begin
  result := Round((FX - Parent.MinX)*Parent.FMultiplier + Parent.Margin);

end;

function TZoomPoint.GetYCoord: Integer;
begin
  result := Parent.FPaintBox.Height -
    Round((FY - Parent.MinY)*Parent.FMultiplier + Parent.Margin);

end;

procedure TZoomPoint.SetX(const Value: Extended);
begin
  FX := Value;
end;

procedure TZoomPoint.SetXCoord(AnX: Integer);
begin
  FX :=  (AnX -  Parent.Margin)/Parent.FMultiplier + Parent.MinX

end;

procedure TZoomPoint.SetY(const Value: Extended);
begin
  FY := Value;
end;

procedure TZoomPoint.SetYCoord(AY: Integer);
begin
   FY := (Parent.FPaintBox.Height - AY -
     Parent.Margin )/Parent.FMultiplier + Parent.MinY ;

end;

function TZoomBox.ReadPBClientHeight: Integer;
begin
  result := FPaintBox.ClientHeight;
end;

function TZoomBox.ReadPBClientWidth: Integer;
begin
  result := FPaintBox.ClientWidth;
end;

function TZoomBox.GetPBClientOrigin: TPoint;
begin
  result := FPaintBox.ClientOrigin;
end;

function TZoomBox.GetPBClientRect: TRect;
begin
  result := FPaintBox.ClientRect;
end;

procedure TZoomBox.SetPBColor(const Value: TColor);
begin
  FPaintbox.Color := Value;
end;

function TZoomBox.GetPBColor: TColor;
begin
  result := FPaintbox.Color;
end;

function TZoomBox.GetPBComponentCount: integer;
begin
  result := FPaintbox.ComponentCount;
end;

function TZoomBox.GetPBComponentIndex: integer;
begin
  result := FPaintbox.ComponentIndex;

end;

procedure TZoomBox.SetPBComponentIndex(const Value: integer);
begin
  FPaintbox.ComponentIndex := Value;
end;

function TZoomBox.GetPBComponents(Index: integer): TComponent;
begin
  result :=  FPaintbox.Components[Index];
end;

function TZoomBox.GetPBControlState: TControlState;
begin
  result :=  FPaintbox.ControlState;
end;

procedure TZoomBox.SetPBControlState(const Controlstate: TControlState);
begin
  FPaintbox.ControlState := Controlstate;
end;

function TZoomBox.GetPBControlStyle: TControlStyle;
begin
  result :=  FPaintbox.ControlStyle;
end;

procedure TZoomBox.SetPBControlStyle(const ControlStyle: TControlStyle);
begin
  FPaintbox.ControlStyle := ControlStyle;
end;

function TZoomBox.GetPBCursor: TCursor;
begin
  result :=  FPaintbox.Cursor;
end;

procedure TZoomBox.SetPBCursor(const Cursor: TCursor);
begin
  FPaintbox.Cursor := Cursor;
end;

function TZoomBox.GetPBDragCursor: TCursor;
begin
  result := FPaintBox.DragCursor
end;

function TZoomBox.GetPBDragKind: TDragKind;
begin
  result := FPaintBox.DragKind
end;

procedure TZoomBox.SetPBDragCursor(const Value: TCursor);
begin
  FPaintBox.DragCursor := Value
end;

procedure TZoomBox.SetPBDragKind(const Value: TDragKind);
begin
  FPaintBox.DragKind := Value
end;

function TZoomBox.GetPBDragMode: TDragMode;
begin
  result := FPaintBox.DragMode;
end;

procedure TZoomBox.SetPBDragMode(const Value: TDragMode);
begin
  FPaintBox.DragMode := Value;
end;

function TZoomBox.GetPBEnabled: Boolean;
begin
  result := FPaintBox.Enabled;
end;

procedure TZoomBox.SetPBEnabled(const Value: Boolean);
begin
  FPaintBox.Enabled := Value;
end;

function TZoomBox.GetPBFont: TFont;
begin
  result := FPaintBox.Font;
end;

procedure TZoomBox.SetPBFont(const Value: TFont);
begin
  FPaintBox.Font := Value;
end;

function TZoomBox.GetPBHeight: Integer;
begin
  result := FPaintBox.Height;
end;

procedure TZoomBox.SetPBHeight(const Value: Integer);
begin
  FPaintBox.Height := Value;
end;

function TZoomBox.GetPBHint: string;
begin
  result := FPaintBox.Hint;
end;

procedure TZoomBox.SetPBHint(const Value: string);
begin
  FPaintBox.Hint := Value;
end;

function TZoomBox.GetPBLeft: Integer;
begin
  result := FPaintBox.Left;
end;

procedure TZoomBox.SetPBLeft(const Value: Integer);
begin
  FPaintBox.Left := Value;
end;

function TZoomBox.GetPBPopupMenu: TPopupMenu;
begin
  result := FPaintBox.PopUpMenu;
end;

procedure TZoomBox.SetPBPopupMenu(const Value: TPopupMenu);
begin
  FPaintBox.PopUpMenu := Value;
end;

function TZoomBox.GetPBShowHint: Boolean;
begin
  result := FPaintBox.ShowHint;
end;

procedure TZoomBox.SetPBShowHint(const Value: Boolean);
begin
  FPaintBox.ShowHint := Value;
end;

function TZoomBox.GetPBTop: Integer;
begin
  result := FPaintbox.Top
end;

procedure TZoomBox.SetPBTop(const Value: Integer);
begin
  FPaintbox.Top := Value;
end;

function TZoomBox.GetPBVisible: Boolean;
begin
  result := FPaintbox.Visible
end;

procedure TZoomBox.SetPBVisible(const Value: Boolean);
begin
  FPaintbox.Visible := Value;
end;

function TZoomBox.GetPBWidth: Integer;
begin
  result := FPaintbox.Width
end;

procedure TZoomBox.SetPBWidth(const Value: Integer);
begin
  FPaintbox.Width := Value;
end;

function TZoomBox.GetPBWindowProc: TWndMethod;
begin
  result := FPaintbox.WindowProc
end;

procedure TZoomBox.SetPBWindowProc(const Value: TWndMethod);
begin
  FPaintbox.WindowProc := Value;
end;

function TZoomBox.GetPBComObject: IUnknown;
begin
  result := FPaintbox.ComObject;
end;

function TZoomBox.GetPBTag: Longint;
begin
  result := FPaintBox.Tag;
end;

procedure TZoomBox.SetPBTag(const Value: Integer);
begin
  FPaintBox.Tag := Value; 
end;

function TZoomBox.GetSBrush: TBrush;
begin
  result := FZoomWindow.Brush
end;

procedure TZoomBox.SetSBrush(const Value: TBrush);
begin
  FZoomWindow.Brush := Value;
end;

function TZoomBox.GetSPen: TPen;
begin
  result := FZoomWindow.Pen;
end;

procedure TZoomBox.SetSPen(const Value: TPen);
begin
  FZoomWindow.Pen := Value;
end;

function TZoomBox.GetSBoundsRect: TRect;
begin
  result := FZoomWindow.BoundsRect;
end;

procedure TZoomBox.SetSBoundsRect(const Value: TRect);
begin
  FZoomWindow.BoundsRect := Value;
end;

function TZoomBox.GetSCursor: TCursor;
begin
  result := FZoomWindow.Cursor;
end;

procedure TZoomBox.SetSCursor(const Value: TCursor);
begin
  FZoomWindow.Cursor := Value;
end;

function TZoomBox.GetSDragCursor: TCursor;
begin
  result := FZoomWindow.DragCursor;
end;

procedure TZoomBox.SetSDragCursor(const Value: TCursor);
begin
  FZoomWindow.DragCursor := Value;
end;

function TZoomBox.GetOnPBPaint: TNotifyEvent;
begin
  result := FPaintBox.OnPaint
end;

procedure TZoomBox.SetOnPBPaint(const Value: TNotifyEvent);
begin
  FPaintBox.OnPaint := Value;
end;

procedure TZoomBox.SetOnClick(const Value: TNotifyEvent);
begin
  FOnClick := Value;
  FPaintBox.OnClick := Value;
end;

{function TZoomBox.GetOnPBDblClick: TNotifyEvent;
begin
  result := FPaintBox.OnDblClick
end;}

procedure TZoomBox.SetOnDblClick(const Value: TNotifyEvent);
begin
  FOnDblClick := Value;
  FPaintBox.OnDblClick := Value;
end;

{function TZoomBox.GetOnPBDragDrop: TDragDropEvent;
begin
  result := FPaintBox.onDragDrop;
end;}

procedure TZoomBox.SetOnDragDrop(const Value: TDragDropEvent);
begin
  FOnDragDrop := Value;
  FPaintBox.onDragDrop := Value;
  FZoomWindow.OnDragDrop := Value;
end;

{function TZoomBox.GetOnPBDragOver: TDragOverEvent;
begin
  result := FPaintBox.OnDragOver;
end;}

procedure TZoomBox.SetOnDragOver(const Value: TDragOverEvent);
begin
  FOnDragOver := Value;
  FPaintBox.OnDragOver := Value;
  FZoomWindow.OnDragOver := Value;
end;

{function TZoomBox.GetOnPBEndDrag: TEndDragEvent;
begin
  result := FPaintBox.OnEndDrag;
end;  }

procedure TZoomBox.SetOnEndDrag(const Value: TEndDragEvent);
begin
  FOnEndDrag := Value;
  FPaintBox.OnEndDrag := Value;
  FZoomWindow.OnEndDrag := Value;
end;

{function TZoomBox.GetOnPBMouseDown: TMouseEvent;
begin
  result := FPaintBox.OnMouseDown;
end; }

procedure TZoomBox.SetOnMouseDown(const Value: TMouseEvent);
begin
  FOnMouseDown := Value;
  FPaintBox.OnMouseDown := Value;
  FZoomWindow.OnMouseDown := Value;
end;

{function TZoomBox.GetOnPBMouseMove: TMouseMoveEvent;
begin
  result := FPaintBox.OnMouseMove;
end; }

procedure TZoomBox.SetOnMouseMove(const Value: TMouseMoveEvent);
begin
  FOnMouseMove := Value;
  FPaintBox.OnMouseMove := Value;
  FZoomWindow.OnMouseMove := Value;
end;

{function TZoomBox.GetOnPBMouseUp: TMouseEvent;
begin
  result := FPaintBox.OnMouseUp;
end; }

procedure TZoomBox.SetOnMouseUp(const Value: TMouseEvent);
begin
  FOnMouseUp := Value;
  FPaintBox.OnMouseUp := Value;
  FZoomWindow.OnMouseUp := Value;
end;

{function TZoomBox.GetOnPBStartDrag: TStartDragEvent;
begin
  result := FPaintBox.OnStartDrag;
end;   }

procedure TZoomBox.SetOnStartDrag(const Value: TStartDragEvent);
begin
  FOnStartDrag := Value;
  FPaintBox.OnStartDrag := Value;
  FZoomWindow.OnStartDrag := Value;
end;

procedure TZoomBox.ZoomOut;
begin
  HorzScrollBar.Position := 0;
  VertScrollBar.Position := 0;
  HorzScrollBar.Visible := False;
  VertScrollBar.Visible := False;
  FPaintBox.Width := ClientWidth;
  FPaintBox.Height:= ClientHeight;
  HorzScrollBar.Range := ClientWidth;
  VertScrollBar.Range := ClientHeight;
  HorzScrollBar.Visible := True;
  VertScrollBar.Visible := True;
  GetMinMax;
  FMultiplier := FDefaultMultiplier;
  FPaintBox.Invalidate;

end;

procedure TZoomBox.GetMinMax;
var
  PointIndex : integer;
  AValue : Extended;
  XMultiplier, YMultiplier : Extended;
  APoint : TZoomPoint;
begin

  if PointsList.Count > 0 then
  begin
    APoint := PointsList.Items[0];
    FMinX := APoint.X;
    FMinY := APoint.Y;
    FMaxX := FMinX;
    FMaxY := FMinY;
    for PointIndex := 0 to PointsList.Count -1 do
    begin
      APoint := PointsList.Items[PointIndex];
      AValue := APoint.X;
      if AValue < MinX then MinX := AValue;
      if AValue > MaxX then MaxX := AValue;
      AValue := APoint.Y;
      if AValue < MinY then MinY := AValue;
      if AValue > MaxY then MaxY := AValue;
    end;
  end
  else
  begin
    FMinX := 0;
    FMaxX := 1;
    FMinY := 0;
    FMaxY := 1;
  end;
  if MaxX = MinX then
  begin
    MaxX := MaxX + 1;
  end;
  if MaxY = MinY then
  begin
    MaxY := MaxY + 1;
  end;
  XMultiplier := (FPaintBox.Width-2*Margin)/(MaxX-MinX);
  YMultiplier := (FPaintBox.Height-2*Margin)/(MaxY-MinY);
  if XMultiplier < YMultiplier
  then
    begin
      FDefaultMultiplier := XMultiplier;
    end
  else
    begin
      FDefaultMultiplier := YMultiplier;
    end;
end;

procedure TZoomBox.SetRange(MinimumX, MaximumX, MinimumY, MaximumY : Extended);
var
  XMultiplier, YMultiplier : Extended;
begin

  FMinX := MinimumX;
  FMaxX := MaximumX;
  FMinY := MinimumY;
  FMaxY := MaximumY;
  if MaxX = MinX then
  begin
    MaxX := MaxX + 1;
  end;
  if MaxY = MinY then
  begin
    MaxY := MaxY + 1;
  end;
  XMultiplier := (FPaintBox.Width-2*Margin)/(MaxX-MinX);
  YMultiplier := (FPaintBox.Height-2*Margin)/(MaxY-MinY);
  if XMultiplier < YMultiplier
  then
    begin
      FMultiplier := XMultiplier;
    end
  else
    begin
      FMultiplier := YMultiplier;
    end;
end;

procedure TZoomBox.SetXRange(MinimumX, MaximumX : Extended);
begin

  FMinX := MinimumX;
  FMaxX := MaximumX;
  if MaxX = MinX then
  begin
    MaxX := MaxX + 1;
  end;
  FMultiplier := (FPaintBox.Width-2*Margin)/(MaxX-MinX);
end;

procedure TZoomBox.SetYRange(MinimumY, MaximumY : Extended);
begin

  FMinY := MinimumY;
  FMaxY := MaximumY;
  if MaxY = MinY then
  begin
    MaxY := MaxY + 1;
  end;
  FMultiplier := (FPaintBox.Height-2*Margin)/(MaxY-MinY);
end;

destructor TZoomBox.Destroy;
begin
  PointsList.Free;
  Inherited Destroy;
end;

procedure TZoomBox.AbortZoom;
begin
  ZoomBegun := False;
  FZoomWindow.Width := 0;
  FZoomWindow.Height := 0;
  FPaintbox.Cursor := crDefault;
end;

procedure TZoomBox.DoExit;
begin
  self.AbortZoom;
  inherited;
//  if Assigned(FOnExit) then FOnExit(Self);
end;

function TZoomBox.SetZoom(AZoomLevel: Extended) : Extended;
var
  TopLeft, BottomRight : TZoomPoint;
begin
  If AZoomLevel <= 0 then raise EInvalidZoomLevel.Create(
    'Error: ZoomLevels must be greater than 0.');
  TopLeft := TZoomPoint.Create(self);
  BottomRight := TZoomPoint.Create(self);
  try
    begin
      TopLeft.XCoord := HorzScrollBar.Position;
      TopLeft.YCoord := VertScrollBar.Position;
      BottomRight.XCoord := HorzScrollBar.Position + ClientWidth;
      BottomRight.YCoord := VertScrollBar.Position + ClientHeight;
      BottomRight.X := (BottomRight.X - TopLeft.X)/AZoomLevel*FMultiplier + TopLeft.X;
      BottomRight.Y := (BottomRight.Y - TopLeft.Y)/AZoomLevel*FMultiplier + TopLeft.Y;
      BeginZoom(TopLeft.XCoord,TopLeft.YCoord);
      FinishZoom(BottomRight.XCoord,BottomRight.YCoord);
      result := FMultiplier;
    end;
  finally
    begin
      TopLeft.Free;
      BottomRight.Free;
    end;
  end;
end;

procedure TZoomBox.ZoomBy(ZoomFactor: Extended);
begin
  SetZoom(ZoomFactor * FMultiplier);
end;

procedure TZoomBox.ZoomByAt(ZoomFactor, X, Y: Extended);
var
  AZoomPoint : TZoomPoint;
begin
  AZoomPoint := TZoomPoint.Create(self);
  AZoomPoint.X := X;
  AZoomPoint.Y := Y;
  try
    begin
      ZoomBy(ZoomFactor);
      HorzScrollBar.Position := AZoomPoint.XCoord - Round(Width/2);
      VertScrollBar.Position := AZoomPoint.YCoord - Round(Height/2);
    end;
  finally
    begin
      AZoomPoint.Free;
    end;
  end;

end;

procedure TZoomBox.MouseToCoordinates(XCoord, YCoord: integer; var X,
  Y: extended);
var
  AZoomPoint : TZoomPoint;
begin
  AZoomPoint := TZoomPoint.Create(self);
  try
    AZoomPoint.XCoord := XCoord;
    AZoomPoint.YCoord := YCoord;
    X := AZoomPoint.X;
    Y := AZoomPoint.Y;
  finally
    AZoomPoint.Free;
  end;

end;

procedure TZoomBox.Invalidate;
begin
  FPaintBox.Invalidate;
end;

procedure TZoomBox.PBBeginDrag(Immediate: Boolean; Threshold: Integer);
begin
  FPaintBox.BeginDrag(Immediate, Threshold);
end;

procedure TZoomBox.PBBringToFront;
begin
  FPaintBox.BringToFront;
end;

function TZoomBox.PBClientToScreen(const Point: TPoint): TPoint;
begin
  result :=  FPaintBox.ClientToScreen( Point);
end;

procedure TZoomBox.PBDragDrop(Source: TObject; X, Y: Integer);
begin
  FPaintBox.DragDrop(Source, X, Y);
end;

function TZoomBox.PBDragging: Boolean;
begin
  result := FPaintBox.Dragging;
end;

procedure TZoomBox.PBEndDrag(Drop: Boolean);
begin
  FPaintBox.EndDrag(Drop);
end;

function TZoomBox.PBGetTextBuf(Buffer: PChar; BufSize: Integer): Integer;
begin
  result := FPaintBox.GetTextBuf(Buffer, BufSize);
end;

function TZoomBox.PBGetTextLen: Integer;
begin
  result := FPaintBox.GetTextLen;
end;

procedure TZoomBox.PBHide;
begin
  FPaintBox.Hide;
end;

function TZoomBox.PBPerform(Msg: Cardinal; WParam,
  LParam: Integer): Longint;
begin
  result := FPaintBox.Perform(Msg, WParam, LParam);
end;

procedure TZoomBox.PbRefresh;
begin
  FPaintBox.Refresh;
end;

procedure TZoomBox.PBRepaint;
begin
  FPaintBox.Repaint;
end;

function TZoomBox.PBScreenToClient(const Point: TPoint): TPoint;
begin
  result := FPaintBox.ScreenToClient(Point);
end;

procedure TZoomBox.PBSendToBack;
begin
  FPaintBox.SendToBack;
end;

procedure TZoomBox.PBSetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  FPaintBox.SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TZoomBox.PBSetTextBuf(Buffer: PChar);
begin
  FPaintBox.SetTextBuf(Buffer);
end;

procedure TZoomBox.PBShow;
begin
  FPaintBox.Show;
end;

procedure TZoomBox.PBUpdate;
begin
  FPaintBox.Update;
end;

procedure TZoomBox.SetOnEndDrop(const Value: TEndDragEvent);
begin
  FOnEndDock := Value;
  FPaintBox.OnEndDock := Value;
  FZoomWindow.OnEndDock := Value;

end;

procedure TZoomBox.SetMultiplier( Value: extended);
var
  NewWidth, NewHeight : Integer;
begin
  if Value <> FMultiplier then
  begin
    NewWidth :=  Trunc(FPaintBox.Width * Value / FMultiplier)+1;
    If NewWidth > High(Smallint) then
    begin
      Value := Value * High(Smallint) / NewWidth
    end;
    NewHeight :=  Trunc(FPaintBox.Height * Value / FMultiplier)+1;
    If NewHeight > High(Smallint) then
    begin
      Value := Value * High(Smallint) / NewHeight
    end;
    PBWidth :=  Trunc(FPaintBox.Width * Value / FMultiplier)+1;
    PBHeight :=  Trunc(FPaintBox.Height * Value / FMultiplier)+1;
    FMultiplier := Value;
  end;
end;

end.
